#!/bin/bash -u

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: statistics.sh,v $
#	$Revision: 1.18 $
#	$Date: 2008/06/09 09:35:35 $
#	$Author: i.zapreev $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2008.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: This script collects and processes the statistical data.
#	@param ${1} the test_list  file
#	@param ${2} the input_list  file
#	@param ${3} the PERFORMANCE, MEMORY, CONFIDENCE or SAMPLE option for ${PERFORMANCE},
#			${MEMORY}, ${CONFIDENCE} or ${SAMPLE} statistics
#	@param ${4} if equals to 0 then we are to remove the previous input statistics
#	NOTE: it is usefull when the performance, confidence, memory and samples statistics
#	has to be generated one after another.
#

#The directory with scripts responcible for generating final simulation statistics from the raw data
SIMULATION_GENERATE_STAT_SCRIPTS="${SIMULATIONS_SHELL_SCRIPTS}/generate_statistics"

#The availbale statistic types
PERFORMANCE="performance"
MEMORY="memory"
CONFIDENCE="confidence"
SAMPLE="sample"

#Get the list of tests
TESTS=`eval cat ${1} | sed -e '/#.*/d'`
#Get the list of inputs
INPUTS=`eval cat ${2} | sed -e '/#.*/d'`

#Read the script parameters, such as the statistics type and etc.
STAT_TYPE=${3}
REMOVE_OLD_STAT=${4}

#Set up the data units
if test "${STAT_TYPE}" == "${PERFORMANCE}"; then
	echo "------------ Collecting PERFORMANCE statistics and preparing data ------------"
	DATA_UNIT=${MILLISECONDS}
	STAT_FILES_EXT="${STAT_TYPE}.statistics"
	GNUPLOT_FILES_EXT="${STAT_TYPE}.gnuplot"
elif test "${STAT_TYPE}" == "${MEMORY}"; then
	echo "------------ Collecting MEMORY statistics and preparing data ------------"
	DATA_UNIT=${KILOBYTES}
	STAT_FILES_EXT="${STAT_TYPE}.statistics"
	GNUPLOT_FILES_EXT="${STAT_TYPE}.gnuplot"
elif test "${STAT_TYPE}" == "${CONFIDENCE}"; then
	echo "------------ Collecting CONFIDENCE statistics and preparing data ------------"
	DATA_UNIT=${CONFUNIT}
	STAT_FILES_EXT="${STAT_TYPE}.statistics"
	GNUPLOT_FILES_EXT="${STAT_TYPE}.gnuplot"
elif test "${STAT_TYPE}" == "${SAMPLE}"; then
	echo "------------ Collecting SAMPLE statistics and preparing data ------------"
	DATA_UNIT="1"	#The number of samples is not scaled
	STAT_FILES_EXT="${STAT_TYPE}.statistics"
	GNUPLOT_FILES_EXT="${STAT_TYPE}.gnuplot"
else
	echo "ERROR: Unknown type of statistics, check the ./performance_tests/simulations/scripts/shell/test_suite.sh script."
	exit 1
fi

####################################################
#%%%%%%%%%%%%EXECUTE DATA PROCESSING%%%%%%%%%%%%%%%#
####################################################

export PATH=.:${PATH}		#Add local directory to the path, otherwise the tests will not run

#Define the statistics folder
export MAIN_STAT_FOLDER=statistics
if ! test -d ${MAIN_STAT_FOLDER}; then
	mkdir ${MAIN_STAT_FOLDER}
fi

#Remove the input-statistic folders if needed
if test ${REMOVE_OLD_STAT} -eq 0; then
	for input in ${INPUTS}
	do
		rm -rf ${MAIN_STAT_FOLDER}/${input}
	done
fi

#Perform data collection and processing for the statistics.
#For every available test case
for test in ${TESTS}
do
	export TEST_NAME=${test}
	echo "${TEST_NAME}: "
	#For every available input
	for input in ${INPUTS}
	do
		export INPUT_DIR=inputs/${input}
		export MAIN_INPUT_STAT_FOLDER=${MAIN_STAT_FOLDER}/${input}
		
		if ! test -d ${MAIN_INPUT_STAT_FOLDER}; then
			mkdir ${MAIN_INPUT_STAT_FOLDER}
		fi
		
		echo "  ${input}:"
		
		#Otain the list of tools that have to be run with this input
		TOOLS=`eval cat ${INPUT_DIR}/tools_list | sed -e '/#.*/d'`
		#For every tool related to the input
		for tool in ${TOOLS}
		do
			#Set the prefix name of the Tool related files for the given input
			export TOOL_PARAMETERS=${INPUT_DIR}/${tool}
			
			#For all different command-line option combinations
			cat ${TOOL_PARAMETERS}/options | sed -e '/#.*/d' | while read options; do
				export OPTS=`eval echo ${options} | sed -e 's/ /./g'`
				#If the list of options is empty then we need to set it to "common"
				if ! test -n "${OPTS}"; then
					OPTS="common"
				fi
				export TEST_CASE_NAME=${TEST_NAME}.${input}.${tool}.${OPTS}
				
				echo -n "    ${TEST_CASE_NAME}"
			
				#Prepare he in and out file prefixes
				export TEST_CASE_STAT_FILE_PREFIX=${TEST_NAME}/${STAT_DIR_NAME}/${input}/${TEST_CASE_NAME}
				export STAT_FILE_PREFIX=${MAIN_INPUT_STAT_FOLDER}/${input}.${tool}.${OPTS}
				#Collect statistics
				. ${SIMULATION_GENERATE_STAT_SCRIPTS}/${STAT_TYPE}_generate_statistics.sh
				
				echo " ..........DONE"
			done
		done
	done
done

echo "Converting the statistics into the gnuplot data files:"
for input in ${INPUTS}
do
	#The input parameters directory
	export INPUT_DIR=inputs/${input}
	
	#Prepare the input parameters for the awk script
	OUT_DAT_FILE_NAME="${MAIN_STAT_FOLDER}/${input}.${STAT_TYPE}"
	#Because this name is going to be used in a sed script we need its escaped version!
	ESCAPED_OUT_DAT_FILE_NAME="${MAIN_STAT_FOLDER}\/${input}.${STAT_TYPE}"
	OUT_DAT_FILE=${OUT_DAT_FILE_NAME}.dat;
	PARAM=${TEST_SUBSUITE_NAME}.param;

	echo "  ${OUT_DAT_FILE}: "
	
	#Remove the old dat file
	rm -f ${OUT_DAT_FILE};
	
	#Place the input files into the arrange_table.awk input
	#in the same order as we have options in the optins_list
	#such that the columns are ordered by Tool/Tool options
	rm -f stat.files.tmp
	
	#Otain the list of tools for this input
	TOOLS=`eval cat ${INPUT_DIR}/tools_list | sed -e '/#.*/d'`
	#For every tool related to the input
	for tool in ${TOOLS}
	do
		#Set the prefix name of the Tool related files for the given input
		TOOL_PARAMETERS=${INPUT_DIR}/${tool}
		
		#For all different command-line option combinations
		cat ${TOOL_PARAMETERS}/options | sed -e '/#.*/d' | while read options; do
			OPTS=`eval echo ${options} | sed -e 's/ /./g'`
			#If the list of options is empty then we need to set it to "common"
			if ! test -n "${OPTS}"; then
				OPTS="common"
			fi
			
			#Generate list of statistic files
			. ${SIMULATION_GENERATE_STAT_SCRIPTS}/${STAT_TYPE}_generate_stat_files_list.sh
		done
	done
	STAT_FILES_LIST=`eval cat stat.files.tmp`
	rm -f stat.files.tmp
	
	#Arrange the dat file with all gnuplot data
	echo "${PARAM} ${STAT_FILES_LIST}" | awk -v out_file_name=${OUT_DAT_FILE} -v normalization_factor=${DATA_UNIT} -f ${PERFORMANCE_AWK_SCRIPTS}/arrange_table.awk
	
	#Get rid of "," which whould be substituted by "."
	#Because awk and gnuplot have different format for real numbers
	cat ${OUT_DAT_FILE} | sed -e 's/,/./g' > ${OUT_DAT_FILE}.tmp
	rm -f ${OUT_DAT_FILE}
	mv ${OUT_DAT_FILE}.tmp ${OUT_DAT_FILE}
	
	#Source the script that defines the data-unit name for the statistics, i.e. DATA_UNIT_STR
	. ${PERFORMANCE_SHELL_SCRIPTS}/${STAT_TYPE}_define_data_unit.sh
	
	#Source the script that produces the MIN/MAX data range in the form of exported variables: MIN MAX
	. ${PERFORMANCE_SHELL_SCRIPTS}/min_max_data_range.sh
	
	#Generate gnuplot ot of the template and then make eps files.
	${PERFORMANCE_SHELL_SCRIPTS}/generate_eps.sh  "${INPUT_DIR}/${GNUPLOT_FILES_EXT}" "${MIN}" "${MAX}" "${ESCAPED_OUT_DAT_FILE_NAME}" "${OUT_DAT_FILE_NAME}" "${DATA_UNIT_STR}"
	
	echo "    --==WE ARE DONE==--"
done
