#!/bin/bash -u

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: test_suite.sh,v $
#	$Revision: 1.22 $
#	$Date: 2008/06/13 07:38:18 $
#	$Author: i.zapreev $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2008.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: This script runs simulation tests
#

#The directory with scripts responcible for the tools invocation
SIMULATION_TOOL_INVOCATION_SCRIPTS="${SIMULATIONS_SHELL_SCRIPTS}/invoke_tools"

#Include the waiting and statistics functions
. ${PERFORMANCE_SHELL_SCRIPTS}/on_wait_statistics_gathering.sh

#Obtain the list of test cases
TESTS=`eval cat ${1} | sed -e '/#.*/d'`
#Otain the list of available inputs
INPUTS=`eval cat ${2} | sed -e '/#.*/d'`

##########################################
#%%%%%%%%%%%%EXECUTE TESTS%%%%%%%%%%%%%%%#
##########################################

export PATH=.:${PATH}		#Add local directory to the path, otherwise the tests will not run

############################################
#Perform tests, with all given input files,#
#   for the given number of repetitions,   #
#         with all given options.          #
############################################

#The test local statistics directory
export STAT_DIR_NAME=statistics

#For every available test case
for test in ${TESTS}
do
	#Set TEST_NAME
	export TEST_NAME=${test}
	#WARNING: Moving to the test-case folder
	cd ${test}
	echo "    ${test}: "
	
	#Create the directory for the statistics if it is not present
	if ! test -d ${STAT_DIR_NAME}; then
		mkdir ${STAT_DIR_NAME}
	fi
	
	#Generate the model and initialize
	echo -n "      Generating the model: .........."
	rm -f ${TEST_NAME}.prism.log.out
	${TEST_NAME}.sh 1>${TEST_NAME}.prism.log.out
	echo "DONE"
	
	echo "      Simulating the test:"
	#For every available input
	for input in ${INPUTS}
	do
		export INPUT_DIR=../inputs/${input}
		echo -n "          ${input}:"
		
		#Create the directory for the statistics if it is not present
		export INPUT_STAT_DIR_NAME=${STAT_DIR_NAME}/${input}
		if ! test -d ${INPUT_STAT_DIR_NAME}; then
			mkdir ${INPUT_STAT_DIR_NAME}
		fi
		
		#Otain the list of tools that have to be run with this input
		TOOLS=`eval cat ${INPUT_DIR}/tools_list | sed -e '/#.*/d'`
		
		#Rerun the experiments the required number of times
		for (( i=0 ; i < ${NUMBER_OF_PERFORMANCE_REPETITIONS} ; i=i+1 )) ; do
			echo -n " ${i}:"
			#For every tool related to the input
			for tool in ${TOOLS}
			do
				export CURR_TOOL_NAME=${tool}
				#Set the prefix name of the Tool related files for the given input
				export TOOL_PARAMETERS=${INPUT_DIR}/${tool}
				
				#Set the input files for the given test
				export TOOL_FILES=`eval . ${TOOL_PARAMETERS}/files`
				
				#For all different command-line option combinations
				cat ${TOOL_PARAMETERS}/options | sed -e '/#.*/d' | while read options; do
					#Export the test-case name.
					OPTS=`eval echo ${options} | sed -e 's/ /./g'`
					#If the list of options is empty then we need to set it to "common"
					if test -n "${OPTS}"; then
						export TEST_CASE_NAME=${TEST_NAME}.${input}.${tool}.${OPTS}
					else
						export TEST_CASE_NAME=${TEST_NAME}.${input}.${tool}.common
					fi
					#We have to export it here for valgrind, since LONG_TEST_CASE_NAME
					#is used for forming the log file name.
					export LONG_TEST_CASE_NAME=${TEST_SUITE_NAME}.${TEST_SUBSUITE_NAME}.${TEST_CASE_NAME}
					
					#Clean up the tmp files and the old .csl files for ymer, if any
					rm -f ${TEST_CASE_NAME}.out.tmp ${TMPDIR}/${input}.${tool}.csl
					
					#Distinguish between all the tools
					TOOL_INVOCATION_SCRIPT=${SIMULATION_TOOL_INVOCATION_SCRIPTS}/${tool}.invocation.sh
					MRMC_TOOL_NAME=mrmc
					if ! test -f ${TOOL_INVOCATION_SCRIPT}; then
						#If we could not find the exact invocation script that we need, then we,
						#in case of MRMC, try to use the regular mrmc.invocation.sh script
						if ! test -f ${TOOL_INVOCATION_SCRIPT}; then
							TOOL_INVOCATION_SCRIPT=${SIMULATION_TOOL_INVOCATION_SCRIPTS}/${tool/#${MRMC_TOOL_NAME}*/${MRMC_TOOL_NAME}}.invocation.sh
							#In this case we should also set the tool's name to be just MRMC. This
							#ensures that the MRMC's statistics extraction scripts will be used.
							export CURR_TOOL_NAME=${MRMC_TOOL_NAME}
						else
							echo -e "\n ERROR: Do not have the tool invocation script for: " ${tool}
							exit
						fi
					fi
					#Invoke the tool, and do not forget to read some common options
					#that does not make it to the test case name
					EXTRA_OPTIONS=`eval cat ${TOOL_PARAMETERS}/common.options`
					. ${TOOL_INVOCATION_SCRIPT}
					
					#Wait until the process ends, here we gather the memory sage statistics
					#WARNING: Since the test is repeated several times the Average-memory usage
					#in statistics will reflectonly the results for the last run of the tools!
					#The peak-memory usage in statistics should be the maximum over all runs
					doWait "${PERFORMANCE_TEST_TIMEOUT_SECS}" "${TOOL_PID}" "${INPUT_STAT_DIR_NAME}/${TEST_CASE_NAME}.memstat"
					#WARNING: THERE HAS TO BE NO OPERATORS BETWEEN doWait and the next script invocation!
					${SIMULATIONS_SHELL_SCRIPTS}/process_test_case_results.sh $?
					
					#Clean up the tmp files and the old .csl files for ymer, if any
					rm -f ${TEST_CASE_NAME}.out.tmp ${TMPDIR}/${input}.${tool}.csl
				done
			done
		done
		echo " DONE"
	done
	
	#Remove the generated tra and lab files
	rm -f ${TMPDIR}/${TEST_NAME}.tra ${TMPDIR}/${TEST_NAME}.lab
	
	#Return to the test directory from the test case
	cd ..
done

if test ${NUMBER_OF_PERFORMANCE_REPETITIONS} -ne  0; then
	${SIMULATIONS_SHELL_SCRIPTS}/statistics.sh ${1} ${2} performance 0
	${SIMULATIONS_SHELL_SCRIPTS}/statistics.sh ${1} ${2} memory 1
	${SIMULATIONS_SHELL_SCRIPTS}/statistics.sh ${1} ${2} sample 1
	${SIMULATIONS_SHELL_SCRIPTS}/statistics.sh ${1} ${2} confidence 1
fi
