#!/bin/bash

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: test_all.sh,v $
#	$Revision: 1.25 $
#	$Date: 2008/06/15 15:58:35 $
#	$Author: i.zapreev $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2006.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: Runs tests.
#

PWD=`eval pwd`
export PATH=${PATH}:.

##########################################################
## Definitions needed for the scripts output formatting ##
##########################################################

#The position after which we will print the PASS/FAIL result
export MAX_RESULT_LENGTH=55

#Note: 12 here is 8 symbold for '\t' and '4' simbolds for "PASS" or "FAIL"
#This is an ugly solution but right now I do not have time for a better one
let "MAX_LINE_LENGTH = MAX_RESULT_LENGTH + 12"
export MAX_LINE_LENGTH
export THE_LINE_SYMBOL="*"

#Create a whole line of symbols here
THE_SYMBOL_LYNE=""
for (( i=1 ; i <= ${MAX_LINE_LENGTH} ; i++ )) ; do
	THE_SYMBOL_LINE="${THE_SYMBOL_LINE}${THE_LINE_SYMBOL}"
done
export THE_SYMBOL_LINE

#####################################
#### Set the following variables ####
#####################################

. ./settings.cfg

#####################################

################
##### MAIN #####
################
HELP_STR="help"
VALGRIND_STR="valgrind"
LINT_STR="lint"
ALL_STR="all"
INTERNAL_STR="internal"
FUNCTIONAL_STR="functional"
PERFORMANCE_STR="performance"

#An internaly used valgrind log options
LOG_TEMPL_NAME="QUALIFIER_NAME"
LOG_PARAMETERS="--time-stamp=yes --log-file=${VALGRIND_LOG_FILES_DIR}/mrmc.log.${LOG_TEMPL_NAME}"

help ()
{
	echo "Usage: ${0}  [options]"
	echo "Options:"
	echo "  -${ALL_STR}: run all tests"
	echo "  -${INTERNAL_STR}: run internal tests"
	echo "  -${FUNCTIONAL_STR}: run functional tests"
	echo "  -${PERFORMANCE_STR}: run performance tests"
	echo "  -${VALGRIND_STR}: turn on Valgrind (mrmc has to be compiled with '-O0 -ggdb -g' options)"
	echo "  -${LINT_STR}: turn on splint checking for internal tests"
}

#Print a note message ${1}
printNote()
{
	echo "${THE_SYMBOL_LINE}"
	echo "${THE_LINE_SYMBOL} NOTE: ${1}"
	echo "*"
}

#Valgrind is a memory profiler
valgrind ()
{
	printNote "Valgrind is ON"
	
	export MRMC="${VALGRIND_HOME}/valgrind ${LOG_PARAMETERS} ${EXTRA_VALGRIND_PARAM} ${MRMC}"
	#Test if the log directory exists, if YES - remove it
	if test -d "${VALGRIND_LOG_FILES_DIR}"; then
		rm -rf ${VALGRIND_LOG_FILES_DIR};
	fi
	#Create a new clean log directory, we do an extra test
	#because the dir may have not been deleted.
	if test ! -d "${VALGRIND_LOG_FILES_DIR}"; then
		mkdir ${VALGRIND_LOG_FILES_DIR};
	fi
}

#Define the log file suffix template for valgrind if needed
#The only parameter is the env variable name
set_valgrind_log_options ()
{
	#If the valgrind is on
	if test ! -z "${VALGRIND}"; then
		export MRMC=`eval echo ${MRMC} | sed -e "s/${LOG_TEMPL_NAME}/\%q\{${1}\}/"`
	fi
}

internal ()
{
	printNote "Running Internal Tests"
	
	cd internal_tests
	if test ! -z "${LINT}"; then
		make lint
	fi
	./test.sh test_list
	cd ..
}

functional ()
{
	printNote "Running Functional Tests"
	
	#Define the log file suffix template for valgrind
	set_valgrind_log_options LONG_TEST_NAME
	cd functional_tests
	./test.sh test_list
	cd ..
}

performance ()
{
	printNote "Running Performance Tests"
	
	#Define the log file suffix template for valgrind
	set_valgrind_log_options LONG_TEST_CASE_NAME
	echo "Each test will be repeated '${NUMBER_OF_PERFORMANCE_REPETITIONS}' times."
	cd performance_tests
	./test.sh test_list
	cd ..
}

all ()
{
	printNote "All tests mode"
	
	internal;
	functional;
	performance;
}

HELP=""
VALGRIND=""
LINT=""
ALL=""
INTERNAL=""
FUNCTIONAL=""
PERFORMANCE=""

opt="$1"
while test ! -z "$opt" -a -z "${opt##-*}"; do
	opt=${opt#-}
	#echo Param: $opt
	case "$opt" in
		${HELP_STR}) HELP=1;;
		${VALGRIND_STR}) VALGRIND=1;;
		${LINT_STR}) LINT=1;;
		${ALL_STR}) ALL=1 ;;
		${INTERNAL_STR}) INTERNAL=1;;
		${FUNCTIONAL_STR}) FUNCTIONAL=1;;
		${PERFORMANCE_STR}) PERFORMANCE=1;;
		*)	echo "ERROR: An unknown script parameter '-${opt}'";
			help;
			exit 1;
			;;
	esac
	shift
	opt="$1"
done

if test ! -z "${VALGRIND}"; then
	valgrind;
fi

if test ! -z "${ALL}"; then
	all;
else
	if test ! -z "${INTERNAL}"; then
		internal;
	fi
	if test ! -z "${FUNCTIONAL}"; then
		functional;
	fi
	if test ! -z "${PERFORMANCE}"; then
		performance;
	fi
	if test ! -z "${HELP}" -o -z "${INTERNAL}" -a -z "${FUNCTIONAL}" -a -z "${PERFORMANCE}"; then
		help;
	fi
fi
